@@ -1,4 +1,5 @@
 /*
+ *      Copyright (C) 2006 Free Software Foundation
  *      Copyright (C) 2002 Fabio Fiorina
  *
  * This file is part of LIBTASN1.
@@ -40,7 +41,7 @@
   #include <getopt.h>
 #endif
 
-char version_man[] = "asn1Parser (GNU libasn1) " VERSION;
+char version_man[] = "asn1Parser (GNU libtasn1) " VERSION;
 
 char help_man[] = "asn1Parser reads files with ASN1 definitions and\n"
                   "generates a C array to use with libtasn1 functions.\n"
@@ -192,7 +193,7 @@ main(int argc,char *argv[])
    printf("asn1Parser: %s\n",errorDescription);
    break;
  default:
-   printf("libasn1 ERROR: %s\n",libtasn1_strerror(parse_result));
+   printf("libtasn1 ERROR: %s\n",libtasn1_strerror(parse_result));
  }
 
 
